# -*- coding: utf-8 -*-
from django.db.models import Q, QuerySet
from . import utils


class TermQuerySet(QuerySet):

    def fast(self):
        return self.select_related('version').prefetch_related('is_a').all()

    def hpo_term(self, hpo_term):
        """Filters for a HPO term.

        Arguments:
            hpo_term (str): HPO Term.

        Returns:
            QuerySet: CrossReference objects
        """
        return self.fast().filter(
            Q(identifier=utils.normalize_hpo_id(hpo_term)),
        )

    def is_a_terms(self, hpo_terms):
        """Filters for a list of is a HPO terms.

        Arguments:
            hpo_terms (list): List of HPO terms.

        Returns:
            QuerySet: Term objects
        """
        return self.fast().filter(
            Q(is_a__identifier__in=[utils.normalize_hpo_id(x) for x in hpo_terms]),
        )

    def can_be_terms(self, hpo_terms):
        """Filters for a list of can be HPO terms.

        Arguments:
            hpo_terms (list): List of HPO terms.

        Returns:
            QuerySet: Term objects
        """
        return self.fast().filter(
            Q(can_be__identifier__in=[utils.normalize_hpo_id(x) for x in hpo_terms]),
        )


class CrossReferenceQuerySet(QuerySet):

    def fast(self):
        return self.select_related('hpo_term').all()

    def hpo_terms(self, hpo_terms):
        """Filters for a list of HPO terms.

        Arguments:
            hpo_terms (list): List of HPO terms.

        Returns:
            QuerySet: CrossReference objects
        """
        return self.fast().filter(
            Q(hpo_term__identifier__in=[utils.normalize_hpo_id(x) for x in hpo_terms]),
        )


class DiseaseQuerySet(QuerySet):

    def fast(self):
        return self.select_related('version') \
            .select_related('age_of_onset__version') \
            .prefetch_related('hpo_terms__version') \
            .prefetch_related('genes') \
            .all()

    def hpo_terms(self, hpo_terms):
        """Filters for a list of HPO terms.

        Arguments:
            hpo_terms (list): List of HPO terms.

        Returns:
            QuerySet: CrossReference objects
        """
        return self.fast().filter(
            Q(hpo_terms__identifier__in=[utils.normalize_hpo_id(x) for x in hpo_terms]),
        )

    def mode_of_inheritance_terms(self, hpo_terms):
        """Filters for a Mode of Inheritance (using HPO Terms).

        Arguments:
            hpo_terms (list): HPO terms.

        Returns:
            QuerySet: Disease objects
        """
        return self.fast().filter(
            Q(mode_of_inheritance__identifier__in=[utils.normalize_hpo_id(x) for x in hpo_terms]),
        )

    def age_of_onset_terms(self, hpo_terms):
        """Filters for a Age of Onset (using HPO Terms).

        Arguments:
            hpo_terms (list): HPO terms.

        Returns:
            QuerySet: Disease objects
        """
        return self.fast().filter(
            Q(age_of_onset__identifier__in=[utils.normalize_hpo_id(x) for x in hpo_terms]),
        )

    def frequency_terms(self, hpo_terms):
        """Filters for a Frequency (using HPO Terms).

        Arguments:
            hpo_terms (list): HPO terms.

        Returns:
            QuerySet: Disease objects
        """
        return self.fast().filter(
            Q(frequency__identifier__in=[utils.normalize_hpo_id(x) for x in hpo_terms]),
        )



# -*- coding: utf-8 -*-
from rest_framework import routers

from . import viewsets


router = routers.SimpleRouter()
router.register(r'', viewsets.TermViewSet)
router.register(r'hpo-xrefs', viewsets.CrossReferenceViewSet)
router.register(r'diseases', viewsets.DiseaseViewSet)
router.register(r'disease-genes', viewsets.DiseaseGeneViewSet, base_name='disease_genes')

default_router = routers.DefaultRouter()
default_router.register(r'hpo-terms', viewsets.TermViewSet)
default_router.register(r'hpo-xrefs', viewsets.CrossReferenceViewSet)
default_router.register(r'diseases', viewsets.DiseaseViewSet)
default_router.register(r'disease-genes', viewsets.DiseaseGeneViewSet, base_name='disease_genes')


urlpatterns = default_router.urls



# -*- coding: utf-8
from django.contrib import admin

from . import models


class TermAdmin(admin.ModelAdmin):
    model = models.Term
    list_display = ('version', 'hpo_term', 'label', 'created_by', 'created', 'modified')
    raw_id_fields = ('version', 'is_a', 'can_be')
    search_fields = ('version__label', 'identifier', 'label', 'description', 'synonyms__description')
    save_as = True


class VersionAdmin(admin.ModelAdmin):
    model = models.Version
    list_display = ('label', 'created', 'modified')
    search_fields = ('label', )
    save_as = True


class SynonymAdmin(admin.ModelAdmin):
    model = models.Synonym
    list_display = ('hpo_term', 'scope', 'created', 'modified')
    list_filter = ('scope', )
    search_fields = ('description', )
    save_as = True


class CrossReferenceAdmin(admin.ModelAdmin):
    model = models.CrossReference
    list_display = ('hpo_term', 'source', 'source_value', 'created', 'modified')
    list_filter = ('source', )
    search_fields = ('source_value', )
    save_as = True


class DiseaseAdmin(admin.ModelAdmin):
    model = models.Disease
    list_display = (
        'label', 'qualifier', 'evidence_code',
        'mode_of_inheritance', 'age_of_onset', 'frequency',
        'create_date', 'created', 'modified'
    )
    raw_id_fields = ('mode_of_inheritance', 'age_of_onset', 'frequency', 'hpo_terms')
    list_filter = ('database', 'qualifier', 'evidence_code')
    search_fields = (
        'description',
        'hpo_terms__label',
        'hpo_terms__description',
        'hpo_terms__synonyms__description'
    )
    save_as = True


admin.site.register(models.Term, TermAdmin)
admin.site.register(models.Version, VersionAdmin)
admin.site.register(models.Synonym, SynonymAdmin)
admin.site.register(models.CrossReference, CrossReferenceAdmin)
admin.site.register(models.Disease, DiseaseAdmin)



from django.conf import settings


hpo_purl = 'http://purl.obolibrary.org/obo/hp.obo'
HPO_URL = getattr(settings, 'HPO_URL', hpo_purl)

hpo_diseases = 'http://compbio.charite.de/jenkins/job/hpo.annotations/'
hpo_diseases += 'lastSuccessfulBuild/artifact/misc/phenotype_annotation.tab'
HPO_DISEASES_URL = getattr(settings, 'HPO_PHENOTYPES_URL', hpo_diseases)

hpo_disease_genes = 'http://compbio.charite.de/jenkins/job/hpo.annotations.monthly/'
hpo_disease_genes += 'lastSuccessfulBuild/artifact/annotation/genes_to_diseases.txt'
HPO_DISEASE_GENES_URL = getattr(settings, 'HPO_DISEASE_GENES_URL', hpo_disease_genes)



# -*- coding: utf-8 -*-
from django.core.validators import validate_comma_separated_integer_list
from django.db import models
from django.utils.translation import ugettext_lazy as _

from model_utils.models import TimeStampedModel

from . import choices, managers


class Term(models.Model):
    """HPO Terms.

    Source: http://purl.obolibrary.org/obo/hp.obo
    """

    version = models.ForeignKey('hpo_terms.Version', on_delete=models.CASCADE)
    identifier = models.PositiveIntegerField(db_index=True)
    label = models.CharField(max_length=255, blank=True)
    description = models.TextField(blank=True)
    created_by = models.CharField(max_length=50, blank=True)
    created = models.CharField(max_length=25, blank=True)
    modified = models.DateTimeField(auto_now=True)

    is_a = models.ManyToManyField('self', blank=True)
    can_be = models.ManyToManyField('self', blank=True)
    alternate_ids = models.CharField(
        max_length=500,
        blank=True,
        validators=[validate_comma_separated_integer_list]
    )

    objects = managers.TermQuerySet.as_manager()

    class Meta:
        verbose_name = _('Term')
        verbose_name_plural = _('Terms')
        index_together = [
            ['version', 'identifier']
        ]

    def __str__(self):
        return self.hpo_term

    @property
    def hpo_term(self):
        return 'HP:{0}'.format(str(self.identifier).zfill(7))

    @property
    def hpo_url(self):
        return 'http://compbio.charite.de/hpoweb/showterm?id={0}'.format(self.hpo_term)


class Version(TimeStampedModel):
    """Model to keep track of HPO version."""

    label = models.CharField(max_length=100)

    class Meta:
        verbose_name = _('Version')
        verbose_name_plural = _('Versions')

    def __str__(self):
        return self.label


class Synonym(TimeStampedModel):
    """Synonms for HPO terms."""

    hpo_term = models.ForeignKey(
        'hpo_terms.Term',
        related_name='synonyms',
        blank=True,
        null=True,
        on_delete=models.CASCADE,
    )
    description = models.TextField(blank=True)
    scope = models.PositiveSmallIntegerField(choices=choices.SYNONYM_SCOPES, blank=True, null=True)

    class Meta:
        verbose_name = _('Synonym')
        verbose_name_plural = _('Synonyms')

    def __str__(self):
        return str(self.hpo_term)


class CrossReference(TimeStampedModel):
    """XRefs for HPO terms."""

    hpo_term = models.ForeignKey(
        'hpo_terms.Term',
        related_name='xrefs',
        blank=True,
        null=True,
        on_delete=models.CASCADE,
    )
    source = models.PositiveSmallIntegerField(choices=choices.CROSS_REFERENCES)
    source_value = models.CharField(max_length=150)

    objects = managers.CrossReferenceQuerySet.as_manager()

    class Meta:
        verbose_name = _('XRef')
        verbose_name_plural = _('XRefs')

    def __str__(self):
        return str(self.hpo_term)


class Disease(TimeStampedModel):
    """A known disorder description. Stores OMIM and ORPHA terms.

    Note:
        Sources:
        - http://compbio.charite.de/jenkins/job/hpo.annotations/
        lastSuccessfulBuild/artifact/misc/phenotype_annotation.tab
        - http://compbio.charite.de/jenkins/job/hpo.annotations.monthly/
        lastSuccessfulBuild/artifact/annotation/genes_to_diseases.txt
    """

    version = models.ForeignKey('hpo_terms.Version', on_delete=models.CASCADE)
    database = models.PositiveSmallIntegerField(choices=choices.DATABASES)
    identifier = models.IntegerField()
    description = models.TextField(blank=True)
    qualifier = models.PositiveSmallIntegerField(choices=choices.QUALIFIERS, blank=True, null=True)
    evidence_code = models.PositiveSmallIntegerField(choices=choices.EVIDENCE, blank=True, null=True)
    mode_of_inheritance = models.ForeignKey(
        'hpo_terms.Term',
        on_delete=models.SET_NULL,
        related_name='disease_mois',
        blank=True,
        null=True,
    )
    age_of_onset = models.ForeignKey(
        'hpo_terms.Term',
        on_delete=models.SET_NULL,
        related_name='disease_age_of_onsets',
        blank=True,
        null=True,
    )
    frequency = models.ForeignKey(
        'hpo_terms.Term',
        on_delete=models.SET_NULL,
        related_name='disease_frequencies',
        blank=True,
        null=True,
    )
    create_date = models.DateField()
    assigned_by = models.CharField(max_length=255)
    hpo_terms = models.ManyToManyField(
        'hpo_terms.Term',
        related_name='diseases',
        blank=True,
    )
    genes = models.ManyToManyField(
        'genome.Gene',
        related_name='diseases',
        blank=True,
    )

    objects = managers.DiseaseQuerySet.as_manager()

    class Meta:
        verbose_name = _('Disease')
        verbose_name_plural = _('Diseases')
        unique_together = ('version', 'database', 'identifier')
        index_together = [
            ['database', 'identifier']
        ]

    def __str__(self):
        return self.label

    @property
    def label(self):
        return '{0}:{1}'.format(self.get_database_display(), self.identifier)

    @property
    def database_url(self):
        if self.get_database_display() == 'OMIM':
            return 'https://www.omim.org/entry/{0}'.format(str(self.identifier).zfill(6))
        elif self.get_database_display() == 'ORPHA':
            return 'http://www.orpha.net/consor/cgi-bin/OC_Exp.php?Lng=EN&Expert={0}'.format(str(self.identifier))
        elif self.get_database_display() == 'DECIPHER':
            return 'https://decipher.sanger.ac.uk/syndrome/{0}#overview'.format(str(self.identifier))

    @property
    def hpo_mode_of_inheritance_url(self):
        return self.mode_of_inheritance.hpo_url

    @property
    def hpo_age_of_onset_url(self):
        return self.age_of_onset.hpo_url

    @property
    def hpo_frequency_url(self):
        return self.frequency.hpo_url



# -*- coding: utf-8 -*-
from genomix.fields import DisplayChoiceField
from rest_framework import serializers

from . import choices, models


class TermSerializer(serializers.ModelSerializer):
    """Serializer for HPO Terms."""

    version = serializers.StringRelatedField()

    class Meta:
        model = models.Term
        fields = (
            'id', 'version', 'hpo_term', 'label', 'description',
            'hpo_url',
            'created_by', 'created', 'modified',
        )


class SynonymSerializer(serializers.ModelSerializer):
    """Serializer for HPO Term Synonms."""

    hpo_term = serializers.StringRelatedField()
    scope = DisplayChoiceField(choices=choices.SYNONYM_SCOPES)

    class Meta:
        model = models.Term
        fields = (
            'id', 'hpo_term', 'description', 'scope',
            'created', 'modified',
        )


class CrossReferenceSerializer(serializers.ModelSerializer):
    """Serializer for HPO Term Cross References."""

    hpo_term = serializers.StringRelatedField()
    source = DisplayChoiceField(choices=choices.CROSS_REFERENCES)

    class Meta:
        model = models.CrossReference
        fields = (
            'id', 'hpo_term', 'source', 'source_value',
            'created', 'modified',
        )


class DiseaseSerializer(serializers.ModelSerializer):
    """Serializer for Diseases."""

    version = serializers.StringRelatedField()
    database = DisplayChoiceField(choices=choices.DATABASES)
    qualifier = DisplayChoiceField(choices=choices.QUALIFIERS)
    evidence_code = DisplayChoiceField(choices=choices.EVIDENCE)
    mode_of_inheritance = TermSerializer(read_only=True)
    age_of_onset = TermSerializer(read_only=True)
    frequency = TermSerializer(read_only=True)
    hpo_terms = serializers.SerializerMethodField()
    genes = serializers.StringRelatedField(many=True)

    class Meta:
        model = models.Disease
        fields = (
            'id', 'version', 'database', 'identifier', 'description', 'database_url',
            'qualifier', 'evidence_code',
            'mode_of_inheritance', 'age_of_onset', 'frequency',
            'hpo_terms', 'genes',
            'create_date', 'assigned_by',
            'created', 'modified',
        )

    def get_hpo_terms(self, obj):
        return ['{0} - {1}'.format(x.hpo_term, x.label) for x in obj.hpo_terms.all()]



# -*- coding: utf-8 -*-
from django import forms
from django.db.models import CharField, TextField, Q

import django_filters
from genome.models import Gene
from genome.choices import HGNC_GENE_STATUS
from genomix.filters import DisplayChoiceFilter

from . import choices, models, utils


class TermFilter(django_filters.rest_framework.FilterSet):

    hpo_term = django_filters.CharFilter(
        label='HPO Term',
        method='filter_hpo_term',
    )
    is_a = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Term.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )
    is_a_terms = django_filters.BaseCSVFilter(
        label='Is a terms',
        method='filter_is_a_terms',
        widget=django_filters.widgets.CSVWidget
    )
    can_be = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Term.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )
    can_be_terms = django_filters.BaseCSVFilter(
        label='Can be terms',
        method='filter_can_be_terms',
        widget=django_filters.widgets.CSVWidget
    )

    class Meta:
        model = models.Term
        fields = [
            'version',
            'version__label',
            'identifier',
            'label',
            'description',
            'is_a',
            'can_be',
            'diseases__description'
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            },
            TextField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            }
        }

    def filter_hpo_term(self, queryset, name, value):
        return queryset.hpo_term(hpo_term=value)

    def filter_is_a_terms(self, queryset, name, value):
        return queryset.is_a_terms(hpo_terms=value)

    def filter_can_be_terms(self, queryset, name, value):
        return queryset.can_be_terms(hpo_terms=value)


class CrossReferenceFilter(django_filters.rest_framework.FilterSet):

    hpo_term = django_filters.ModelChoiceFilter(
        queryset=models.Term.objects.fast(),
        widget=forms.NumberInput,
    )
    hpo_terms = django_filters.BaseCSVFilter(
        label='HPO Terms',
        method='filter_hpo_terms',
        widget=django_filters.widgets.CSVWidget
    )
    source = DisplayChoiceFilter(choices=choices.CROSS_REFERENCES)

    class Meta:
        model = models.CrossReference
        fields = [
            'hpo_term',
            'source',
            'source_value',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            }
        }

    def filter_hpo_terms(self, queryset, name, value):
        return queryset.hpo_terms(hpo_terms=value)


class DiseaseFilter(django_filters.rest_framework.FilterSet):

    database = DisplayChoiceFilter(choices=choices.DATABASES)
    qualifier = DisplayChoiceFilter(choices=choices.QUALIFIERS)
    evidence_code = DisplayChoiceFilter(choices=choices.EVIDENCE)
    mode_of_inheritance = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Term.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )
    mode_of_inheritance_terms = django_filters.BaseCSVFilter(
        label='Mode of inheritance terms',
        method='filter_mode_of_inheritance_terms',
        widget=django_filters.widgets.CSVWidget
    )
    age_of_onset = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Term.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )
    age_of_onset_terms = django_filters.BaseCSVFilter(
        label='Age of onsent terms',
        method='filter_age_of_onset_terms',
        widget=django_filters.widgets.CSVWidget
    )
    frequency = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Term.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )
    frequency_terms = django_filters.BaseCSVFilter(
        label='Frequency terms',
        method='filter_frequency_terms',
        widget=django_filters.widgets.CSVWidget
    )
    hpo_terms = django_filters.BaseCSVFilter(
        label='HPO Terms',
        method='filter_hpo_terms',
        widget=django_filters.widgets.CSVWidget
    )
    genes = django_filters.ModelMultipleChoiceFilter(
        queryset=Gene.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )
    genes__symbol = django_filters.CharFilter(lookup_expr='iexact', distinct=True)

    class Meta:
        model = models.Disease
        fields = [
            'version',
            'version__label',
            'database',
            'identifier',
            'description',
            'qualifier',
            'evidence_code',
            'mode_of_inheritance',
            'age_of_onset',
            'frequency',
            'genes',
            'genes__symbol',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            },
            TextField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            }
        }

    def filter_hpo_terms(self, queryset, name, value):
        return queryset.hpo_terms(hpo_terms=value)

    def filter_mode_of_inheritance_terms(self, queryset, name, value):
        return queryset.mode_of_inheritance_terms(hpo_terms=value)

    def filter_age_of_onset_terms(self, queryset, name, value):
        return queryset.age_of_onset_terms(hpo_terms=value)

    def filter_frequency_terms(self, queryset, name, value):
        return queryset.frequency_terms(hpo_terms=value)


class DiseaseGeneFilter(django_filters.rest_framework.FilterSet):

    symbol = django_filters.CharFilter(lookup_expr='iexact')
    status = DisplayChoiceFilter(choices=HGNC_GENE_STATUS)
    diseases = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Disease.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )
    hpo_terms = django_filters.BaseCSVFilter(
        label='HPO Terms',
        method='filter_hpo_terms',
        widget=django_filters.widgets.CSVWidget,
    )

    class Meta:
        model = Gene
        fields = [
            'symbol',
            'status',
            'active',
            'chromosome',
            'diseases',
        ]

    def filter_hpo_terms(self, queryset, name, value):
        return queryset.filter(
            Q(diseases__hpo_terms__identifier__in=[
                utils.normalize_hpo_id(x)
                for x in value
            ])
        )
